using System;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Core.Serialization;

public class StateEnableSerializer : IEngineTypeYamlSerializer
{
    private const string SERIALIZED_SCALAR_NAME = "Properties";

    public bool Accepts(Type type) => typeof(IStateEnable).IsAssignableFrom(type);

    public object? ReadYaml(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        bool enabled;

        IStateEnable stateEnable;

        parser.Consume<MappingStart>();

        if (parser.Consume<Scalar>().Value.CompareTo(nameof(IStateEnable.Enabled)) != 0)
            throw new();
        enabled = bool.Parse(parser.Consume<Scalar>().Value);

        if (parser.Consume<Scalar>().Value.CompareTo(SERIALIZED_SCALAR_NAME) != 0)
            throw new();
        SerializedClass instanceSerializedClass = (SerializedClass)rootDeserializer(typeof(SerializedClass))!;
        stateEnable = (IStateEnable)instanceSerializedClass.CreateInstance();

        parser.Consume<MappingEnd>();

        stateEnable.Enabled = enabled;

        return stateEnable;
    }

    public void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        IStateEnable stateEnable = (IStateEnable)value!;

        emitter.Emit(new MappingStart());

        emitter.Emit(new Scalar(nameof(IStateEnable.Enabled)));
        emitter.Emit(new Scalar(stateEnable.Enabled.ToString()));

        emitter.Emit(new Scalar(SERIALIZED_SCALAR_NAME));
        serializer(new SerializedClass(stateEnable));

        emitter.Emit(new MappingEnd());
    }
}
