using System;

using Syntriax.Engine.Core.Factory;

using YamlDotNet.Core;
using YamlDotNet.Core.Events;
using YamlDotNet.Serialization;

namespace Syntriax.Engine.Core.Serialization;

public class TypeContainerSerializer : IEngineTypeYamlSerializer
{
    public bool Accepts(Type type) => type == typeof(TypeContainer);

    public object? ReadYaml(IParser parser, Type type, ObjectDeserializer rootDeserializer)
    {
        parser.Consume<MappingStart>();

        if (parser.Consume<Scalar>().Value.CompareTo(nameof(TypeContainer.Type)) != 0)
            throw new ArgumentException($"{nameof(TypeContainer)} mapping must start with {nameof(TypeContainer.Type)}");
        string typeFullName = parser.Consume<Scalar>().Value;

        if (parser.Consume<Scalar>().Value.CompareTo(nameof(TypeContainer.Value)) != 0)
            throw new ArgumentException($"{nameof(TypeContainer)} mapping must end with {nameof(TypeContainer.Type)}");

        object? value = rootDeserializer(TypeFactory.GetType(typeFullName));

        parser.Consume<MappingEnd>();

        return new TypeContainer() { Type = typeFullName, Value = value };
    }

    public void WriteYaml(IEmitter emitter, object? value, Type type, ObjectSerializer serializer)
    {
        TypeContainer? typeContainer = (TypeContainer)value!;

        emitter.Emit(new MappingStart());

        emitter.Emit(new Scalar(nameof(TypeContainer.Type)));
        emitter.Emit(new Scalar(typeContainer.Type));

        emitter.Emit(new Scalar(nameof(TypeContainer.Value)));
        serializer(typeContainer.Value, TypeFactory.GetType(typeContainer.Type));

        emitter.Emit(new MappingEnd());
    }
}
